package Sys::Socket::Error::InetInvalidNetworkAddress;

1;

=head1 Name

Sys::Socket::Error::InetInvalidNetworkAddress - The address is not a valid network address.

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License

