package SPVM::Sys::FileTest;

1;

=head1 Name

SPVM::Sys::FileTest - File Tests

=head1 Description

C<Sys::FileTest> is the class for file tests.

This class is deprecated. Use methods in the L<Sys|SPVM::Sys> class.
