package SPVM::Sys::Socket::AddrinfoLinkedList;

our $VERSION = '0.01';

1;

=head1 Name

SPVM::Sys::Socket::AddrinfoLinkedList - Sys::Socket::AddrinfoLinkedList is a SPVM module

=head1 Usage

  use Sys::Socket::AddrinfoLinkedList;
  
=head1 Description

C<Sys::Socket::AddrinfoLinkedList> is a L<SPVM> module.

=head1 Copyright & License

Copyright (c) 2023 Yuki Kimoto

MIT License

